package mastermind;

import java.util.ArrayList;

public class Mastermind {

    //mastermind.Mastermind variables
    final private char[] colors;
    final private int maxTurns;
    final private int numOfPegs;
    
    //This is the current instances variables
    private char[] correctPegs;
    private int currentTurn = 0;
    private int correctlyPlaced = 0;
    private int correctlyColored = 0;
    private boolean won = false;

    //Constructors
    public Mastermind() {
        this(10, 4, new char[]{'R', 'G', 'B', 'O', 'Y'});
    }
    public Mastermind(int maxTurns, int numOfPegs, char[] colors) {
        this.maxTurns = maxTurns;
        this.numOfPegs = numOfPegs;
        this.colors = colors;

        //Generate the correct pegs
        correctPegs = new char[numOfPegs];
        for (int i = 0; i < numOfPegs; i++) {
            correctPegs[i] = colors[(int) (Math.floor(Math.random() * colors.length))];
        }

        //printPegs(correctPegs);
    }    
    
    //Gameplay functions
    //This will take the users turn using the given variables
    public void takeTurn(char[] guessPegs) {
        
        //Check the guess
        correctlyPlaced = amountCorrectlyPlaced(guessPegs);
        correctlyColored = amountCorrectlyColored(guessPegs);
        
        //Increase the current turn number
        currentTurn++;
        
        //Check if they have won
        if (correctlyPlaced == numOfPegs) {
            won = true;
        }
    }
    
    //Get the number of correctly located pegs
    public int amountCorrectlyPlaced(char[] pegs) {

        int counter = 0;
        for (int i = 0; i < correctPegs.length; i++) {
            if (correctPegs[i] == pegs[i]) {
                counter++;
            }
        }

        return counter;
    }

    //Get the number of correct colors
    public int amountCorrectlyColored(char[] pegs) {

        int counter = 0;

        //Array of the colors not currently checked
        ArrayList<Character> colorsLeft = new ArrayList<>(colors.length);
        for (int i = 0; i < pegs.length; i++) {
            colorsLeft.add(new Character(correctPegs[i]));
        }

        //Loop through the entered pegs and check whether the correct ones
        //contain that color
        for (int i = 0; i < pegs.length; i++) {
            for (int j = 0; j < colorsLeft.size(); j++) {
                if (pegs[i] == colorsLeft.get(j)) {
                    counter++;
                    colorsLeft.remove(j);
                    break;
                }
            }
        }

        return counter;
    }

    //Return a char array from a string entered
    public char[] convertPegString(String pegs) {

        char[] pegArray = pegs.toCharArray();
        for (int i = 0; i < pegArray.length; i++) {
            pegArray[i] = Character.toUpperCase(pegArray[i]);
        }

        return pegArray;
    }

    //This function will make sure that the guess entered is in the correct format
    public boolean correctFormat(char[] guess) {

        //Check for null
        if (guess == null) {
            return false;
        }

        //Check it is of the correct length
        if (guess.length != getNumOfPegs()) {
            return false;
        }

        //Check if the pegs have a miss formatted peg
        char[] colors = getColors();
        for (int i = 0; i < guess.length; i++) {
            boolean contains = false;
            for (int j = 0; j < colors.length; j++) {
                if (Character.toUpperCase(guess[i]) == colors[j]) {
                    contains = true;
                    break;
                }
            }
            if (!contains) {
                return false;
            }
        }

        return true;
    }

    //Print the peg array
    public void printPegs(char[] pegs) {
        for (char peg : pegs) {
            System.out.print(peg + " ");
        }
    }

    //Getters for the gameplay
    public char[] getColors() {
        return colors;
    }
    public int getMaxTurns() {
        return maxTurns;
    }
    public int getNumOfPegs() {
        return numOfPegs;
    }
    public char[] getCorrectPegs() {
        return correctPegs;
    }
    public int getCurrentTurn() {
        return currentTurn;
    }
    public int getCorrectlyPlaced() {
        return correctlyPlaced;
    }
    public int getCorrectlyColored() {
        return correctlyColored;
    }
    public boolean isWon() {
        return won;
    }    

    //MastermindGUI entry method
    public static void main(String[] args) {
        Mastermind mastermind = new Mastermind();
        char[] guess = {'R', 'G', 'Y', 'B'};

        System.out.println("");
        System.out.println(mastermind.amountCorrectlyPlaced(guess));
        System.out.println(mastermind.amountCorrectlyColored(guess));

    }
}
