package mastermind;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

public class MastermindCommandLine {
    
    Mastermind mastermind;
    BufferedReader br;    
    
    //Get the users guess as a string
    public char[] getGuess() {
        
        String guess = new String();
        
        //Open a bufferedreader for the terminal
        try {
            br = new BufferedReader(new InputStreamReader(System.in));
            guess = br.readLine();
        }
        catch(IOException e) {
            System.out.println("There was an IOException");
        }
        
        return mastermind.convertPegString(guess);
    }

    //Start the command line game
    public void start() {
        
        this.mastermind = new Mastermind();
        System.out.println("Master code generated");
        
        int turnNumber = 0;
        int numCorrectlyPlaced = 0;
        int numCorrectColor = 0;
        
        do {            
            System.out.println("Turns Left: " + (mastermind.getMaxTurns() - turnNumber));
            System.out.print("Enter code: ");
            
            //Get the users guess
            char[] guess = getGuess();
            
            //Quit if q is pressed
            if ((new String(guess).toLowerCase().equals("q"))) {
                System.out.println("Quitting...");
                return;
            }
            
            //If they entered an incorrect guess make them do it again
            if (!mastermind.correctFormat(guess)) {
                do {
                    System.out.println("Guess format incorrect! please try again, or press 'q' to exit: ");
                    System.out.print("Enter code: ");
                    guess = getGuess();
                    
                    if ((new String(guess).toLowerCase().equals("q"))) {
                        System.out.println("Quitting...");
                        return;
                    }
                }
                while(!mastermind.correctFormat(guess));
            }
                        
            numCorrectColor = mastermind.amountCorrectlyColored(guess);
            numCorrectlyPlaced = mastermind.amountCorrectlyPlaced(guess);
            
            System.out.println("C:" + numCorrectColor + " P:" + numCorrectlyPlaced);
            if (numCorrectlyPlaced == mastermind.getNumOfPegs()) {
                System.out.println("Congratulations you have won!");
                return;
            }
            
            turnNumber++;
            
        }
        while(turnNumber < mastermind.getMaxTurns());
        
        System.out.println("Unfortunately you have lost!");
    }
    
    public static void main(String[] args) {
        MastermindCommandLine mastermindCommandLine = new MastermindCommandLine();
        mastermindCommandLine.start();
    }
}
