package mastermind.swing;
import mastermind.Mastermind;

import java.awt.Color;

public class MastermindManager {
    
    private MastermindGUI mastermindGui;
    private Mastermind mastermind;
    
    //Constructors
    public MastermindManager(Mastermind mastermind, MastermindGUI masterindGui) {
        this.mastermind = mastermind;
        this.mastermindGui = masterindGui;
    }
    
    //Start the game and handle communication between the gui and game
    //NOTE: THIS MIGHT BE BETTER AS A CALLBACK EVENT SYSTEM
    public void start() {
        
        mastermindGui.start();
        
        //Create the gameplay loop for the game
        do {
         
            while(!mastermindGui.hasGuessStringAvailable()){
                //Do nothing
            }
                        
            //Get the guess string
            String guessString = mastermindGui.getGuessString();
            System.out.println(guessString);
            
            //Make sure that the string is in correct format
            char[] guessPegs = mastermind.convertPegString(guessString);
            if (!mastermind.correctFormat(guessPegs)) {
                
                //Do something graphically and continue
                mastermindGui.resetGuessString();
                continue;
            }
            else {
                
                //Take the turn in mastermind
                mastermind.takeTurn(guessPegs);
                int correctColor = mastermind.getCorrectlyColored();
                int correctPlace = mastermind.getCorrectlyPlaced();
                
                //Show the guess on the screen and the results
                mastermindGui.displayResult(correctColor, correctPlace, mastermind.getCurrentTurn());
                
                //Reset the guess
                mastermindGui.resetGuessString();
            }
        }
        while(!mastermind.isWon() && (mastermind.getCurrentTurn() < mastermind.getMaxTurns()));
        
        //Check for win condition
        if (mastermind.isWon()) {
            mastermindGui.displayMessage("Congratulations you have won!");
            //mastermindGui.stop();
        }
        else {
            mastermindGui.displayMessage("Unfortunately you have lost!");
            //mastermindGui.stop();
        }
    }
    
    //Entry point
    public static void main(String[] args) {
        
        char[] pinColorLetters = new char[]{'R', 'G', 'B', 'O', 'Y', 'P'};
        int maxTurns = 10;
        int pinNumber = 4;
        Mastermind mastermind = new Mastermind(maxTurns, pinNumber, pinColorLetters);

        //Swing gui
        Color[] pinColors = new Color[]{Color.RED, Color.GREEN, Color.BLUE, Color.ORANGE, Color.YELLOW, new Color(150, 0, 255)};
        MastermindGUI gui = new MastermindGUI(pinNumber, pinColorLetters, pinColors);
        MastermindManager manager = new MastermindManager(mastermind, gui);

        manager.start();
    }
}
